@@ -1273,6 +1273,9 @@ grl_pls_file_to_media (GrlMedia            *content,
     g_clear_error (&error);
     g_free (str);
   } else {
+    GTimeVal time;
+    GDateTime *date_time;
+
     mime = g_file_info_get_content_type (info);
 
     if (!media) {
@@ -1291,6 +1294,16 @@ grl_pls_file_to_media (GrlMedia            *content,
         media = grl_media_new ();
       }
       set_media_id_from_file (media, file);
+    } else {
+      if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY &&
+          !GRL_IS_MEDIA_BOX (media)) {
+        char *uri;
+
+        uri = g_file_get_uri (file);
+        GRL_DEBUG ("URI '%s' is a directory but the passed media item is not GrlMediaBox type", uri);
+        g_free (uri);
+        return NULL;
+      }
     }
 
     if (!GRL_IS_MEDIA_BOX (media)) {
@@ -1312,8 +1325,6 @@ grl_pls_file_to_media (GrlMedia            *content,
     g_free (str);
 
     /* Date */
-    GTimeVal time;
-    GDateTime *date_time;
     g_file_info_get_modification_time (info, &time);
     date_time = g_date_time_new_from_timeval_utc (&time);
     grl_media_set_modification_date (media, date_time);
