@@ -212,7 +212,7 @@ static void authorize_flickr_cb (GtkAction *action);
 #endif
 
 static void shutdown_plugins_cb (GtkAction *action);
-static void shutdown_plugins (gboolean ui_active);
+static void shutdown_plugins (void);
 
 static void load_all_plugins_cb (GtkAction *action);
 static void load_all_plugins (void);
@@ -260,7 +260,7 @@ authorize_flickr_cb (GtkAction *action)
 static void
 shutdown_plugins_cb (GtkAction *action)
 {
-  shutdown_plugins (TRUE);
+  shutdown_plugins ();
 }
 
 static void
@@ -307,7 +307,7 @@ create_browser_model (void)
 					     G_TYPE_OBJECT,     /* Content */
 					     G_TYPE_INT,        /* Type */
 					     G_TYPE_STRING,     /* Name */
-					     GDK_TYPE_PIXBUF)); /* Icon */
+					     G_TYPE_ICON));     /* Icon */
 }
 
 static GtkTreeModel *
@@ -334,39 +334,19 @@ create_query_combo_model (void)
 					     G_TYPE_OBJECT));   /* source */
 }
 
-static GdkPixbuf *
-load_icon (const gchar *icon_name)
-{
-  GdkScreen *screen;
-  GtkIconTheme *theme;
-  GdkPixbuf *pixbuf;
-  GError *error = NULL;
-
-  screen = gdk_screen_get_default ();
-  theme = gtk_icon_theme_get_for_screen (screen);
-  pixbuf = gtk_icon_theme_load_icon (theme, icon_name, 22, 22, &error);
-
-  if (pixbuf == NULL) {
-    GRL_WARNING ("Failed to load icon %s: %s", icon_name,  error->message);
-    g_error_free (error);
-  }
-
-  return pixbuf;
-}
-
-static GdkPixbuf *
+static GIcon *
 get_icon_for_media (GrlMedia *media)
 {
   if (GRL_IS_MEDIA_BOX (media)) {
-    return load_icon (GTK_STOCK_DIRECTORY);
+    return g_themed_icon_new ("folder");
   } else if (GRL_IS_MEDIA_VIDEO (media)) {
-    return load_icon ("gnome-mime-video");
+    return g_themed_icon_new ("gnome-mime-video");
   } else if (GRL_IS_MEDIA_AUDIO (media)) {
-    return load_icon ("gnome-mime-audio");
+    return g_themed_icon_new ("gnome-mime-audio");
   } else if (GRL_IS_MEDIA_IMAGE (media)) {
-    return load_icon ("gnome-mime-image");
+    return g_themed_icon_new ("gnome-mime-image");
   } else {
-    return load_icon (GTK_STOCK_FILE);
+    return g_themed_icon_new ("text-x-generic");
   }
 }
 
@@ -467,6 +447,9 @@ set_cur_resolve (GrlSource *source, GrlMedia *media)
 static void
 clear_panes (void)
 {
+  /* Prevent undesired cursor-changed signal calls */
+  gtk_tree_view_set_model (GTK_TREE_VIEW (view->browser),
+                           NULL);
   if (view->browser_model) {
     gtk_list_store_clear (GTK_LIST_STORE (view->browser_model));
     g_object_unref (view->browser_model);
@@ -681,7 +664,7 @@ browse_search_query_cb (GrlSource *source,
   gint type;
   const gchar *name;
   GtkTreeIter iter;
-  GdkPixbuf *icon;
+  GIcon *icon;
   OperationState *state = (OperationState *) user_data;
   guint next_op_id;
 
@@ -895,16 +878,118 @@ browser_activated_cb (GtkTreeView *tree_view,
   }
 }
 
+static void
+add_source_metadata (GtkTreeModel *model,
+		     const char *key,
+		     const char *value)
+{
+  GtkTreeIter iter;
+
+  gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+  gtk_list_store_set (GTK_LIST_STORE (model),
+                      &iter,
+                      METADATA_MODEL_NAME, key,
+                      METADATA_MODEL_VALUE, value,
+                      -1);
+  GRL_DEBUG ("  %s: %s", key, value);
+
+}
+
+static char *
+media_type_to_str (GrlMediaType type)
+{
+  GString *s;
+
+  if (type == GRL_MEDIA_TYPE_NONE)
+    return g_strdup ("None");
+  if (type == GRL_MEDIA_TYPE_ALL)
+    return g_strdup ("All");
+
+  s = g_string_new (NULL);
+  if (GRL_MEDIA_TYPE_AUDIO & type)
+    g_string_append (s, "audio, ");
+  if (GRL_MEDIA_TYPE_VIDEO & type)
+    g_string_append (s, "video, ");
+  if (GRL_MEDIA_TYPE_IMAGE & type)
+    g_string_append (s, "image, ");
+
+  g_string_truncate (s, s->len - 2);
+  return g_string_free (s, FALSE);
+}
+
+static void
+populate_source_metadata (GrlSource *source)
+{
+  if (view->metadata_model) {
+    gtk_list_store_clear (GTK_LIST_STORE (view->metadata_model));
+    g_object_unref (view->metadata_model);
+  }
+  view->metadata_model = create_resolve_model ();
+  gtk_tree_view_set_model (GTK_TREE_VIEW (view->metadata),
+			   view->metadata_model);
+
+  if (source) {
+    const char *str_props[] = {
+      "source-desc",
+      "source-id",
+      "source-name"
+    };
+    guint i;
+    char *str;
+    guint auto_split_threshold;
+    int rank;
+    GrlMediaType supported_media;
+    GIcon *icon;
+
+    for (i = 0; i < G_N_ELEMENTS (str_props); i++) {
+      g_object_get (G_OBJECT (source), str_props[i], &str, NULL);
+      add_source_metadata (view->metadata_model, str_props[i], str);
+      g_free (str);
+    }
+
+    g_object_get (G_OBJECT (source),
+                  "auto-split-threshold", &auto_split_threshold,
+                  "rank", &rank,
+                  "supported-media", &supported_media,
+                  "source-icon", &icon,
+                  NULL);
+
+    str = g_strdup_printf ("%i", auto_split_threshold);
+    add_source_metadata (view->metadata_model, "auto-split-threshold", str);
+    g_free (str);
+
+    str = g_strdup_printf ("%d", rank);
+    add_source_metadata (view->metadata_model, "rank", str);
+    g_free (str);
+
+    str = media_type_to_str (supported_media);
+    add_source_metadata (view->metadata_model, "supported-media", str);
+    g_free (str);
+
+    if (icon) {
+      str = g_icon_to_string (icon);
+      add_source_metadata (view->metadata_model, "source-icon", str);
+      g_free (str);
+      g_object_unref (icon);
+    }
+  }
+
+  gtk_widget_set_sensitive (view->show_btn, FALSE);
+  ui_state->last_url = NULL;
+}
+
 static void
 resolve (GrlSource *source, GrlMedia *media)
 {
-  if (source) {
+  if (source && media) {
     grl_source_resolve (source,
                         media,
                         all_keys (),
                         default_resolve_options,
                         resolve_cb,
                         NULL);
+  } else {
+    populate_source_metadata (source);
   }
 }
 
@@ -1698,6 +1783,7 @@ set_filesystem_config (void)
 
   config2 = grl_config_new ("grl-filesystem", NULL);
   grl_config_set_string (config2, "base-uri", "file:///");
+  grl_config_set_boolean (config2, "handle-pls", TRUE);
 
   registry = grl_registry_get_default ();
   grl_registry_add_config (registry, config1, NULL);
@@ -1888,7 +1974,7 @@ ui_setup (void)
 
   gint i;
   GtkCellRenderer *col_renders[2];
-  gchar *col_attributes[] = {"pixbuf", "text"};
+  gchar *col_attributes[] = {"gicon", "text"};
   gint col_model[2] = { BROWSER_MODEL_ICON, BROWSER_MODEL_NAME};
   col_renders[0] = gtk_cell_renderer_pixbuf_new ();
   col_renders[1] = gtk_cell_renderer_text_new ();
@@ -1999,10 +2085,11 @@ show_browsable_sources ()
       sources_iter = g_list_next (sources_iter)) {
     GrlSource *source;
     const gchar *name;
-    GdkPixbuf *icon;
+    GIcon *icon;
 
     source = GRL_SOURCE (sources_iter->data);
-    icon = load_icon (GTK_STOCK_DIRECTORY);
+
+    icon = g_themed_icon_new ("folder");
     name = grl_source_get_name (source);
     GRL_DEBUG ("Loaded source: '%s'", name);
     gtk_list_store_append (GTK_LIST_STORE (view->browser_model), &iter);
@@ -2014,9 +2101,7 @@ show_browsable_sources ()
 			BROWSER_MODEL_NAME, name,
 			BROWSER_MODEL_ICON, icon,
 			-1);
-    if (icon) {
-      g_object_unref (icon);
-    }
+    g_object_unref (icon);
   }
   g_list_free (sources);
 }
@@ -2192,7 +2277,7 @@ load_plugins (void)
 }
 
 static void
-shutdown_plugins (gboolean ui_active)
+shutdown_plugins (void)
 {
   GList *plugins;
   GList *plugin_iter;
@@ -2201,10 +2286,7 @@ shutdown_plugins (gboolean ui_active)
   /* Cancel previous operation, if any */
   cancel_current_operation ();
 
-  /* Let's make sure we don't have references to stuff
-     we are about to shut down */
-  if (ui_active)
-    clear_ui ();
+  clear_ui ();
 
   registry = grl_registry_get_default ();
 
@@ -2228,11 +2310,9 @@ shutdown_plugins (gboolean ui_active)
 				     NULL);
 
   /* Reload UI */
-  if (ui_active) {
-    reset_ui ();
-    search_combo_setup ();
-    query_combo_setup ();
-  }
+  reset_ui ();
+  search_combo_setup ();
+  query_combo_setup ();
 }
 
 static void
@@ -2269,6 +2349,7 @@ main (int argc, gchar *argv[])
   configure_plugins ();
   load_plugins ();
   gtk_main ();
-  shutdown_plugins (FALSE);
+  clear_ui ();
+  grl_deinit ();
   return 0;
 }
